
package shopping.cart

import akka.actor.typed.ActorSystem
import akka.http.scaladsl.model.{ HttpRequest, HttpResponse }
import akka.persistence.query.typed
import akka.persistence.query.typed.EventEnvelope
import akka.persistence.typed.PersistenceId
import akka.projection.grpc.producer.EventProducerSettings
import akka.projection.grpc.producer.scaladsl.EventProducer
import akka.projection.grpc.producer.scaladsl.EventProducer.Transformation

import scala.concurrent.Future

object PublishEventsGrpc {

  def eventProducerService(system: ActorSystem[_])
      : PartialFunction[HttpRequest, Future[HttpResponse]] = {
    val transformation = Transformation.identity
      .registerEnvelopeMapper[ShoppingCart.ItemAdded, proto.ItemAdded](
        envelope => Some(transformItemUpdated(envelope)))
      .registerEnvelopeMapper[
        ShoppingCart.ItemQuantityAdjusted,
        proto.ItemQuantityAdjusted](envelope =>
        Some(transformItemAdjusted(envelope)))
      .registerEnvelopeMapper[ShoppingCart.ItemRemoved, proto.ItemRemoved](
        envelope => Some(transformItemRemoved(envelope)))
      .registerEnvelopeMapper[ShoppingCart.CheckedOut, proto.CheckedOut](
        envelope => Some(transformCheckedOut(envelope)))

    val eventProducerSource = EventProducer.EventProducerSource(
      "ShoppingCart",
      "cart",
      transformation,
      EventProducerSettings(system))

    EventProducer.grpcServiceHandler(eventProducerSource)(system)
  }
  

  
  private def transformItemUpdated(
      envelope: EventEnvelope[ShoppingCart.ItemAdded]): proto.ItemAdded = {
    val event = envelope.event
    proto.ItemAdded(
      cartId = PersistenceId.extractEntityId(envelope.persistenceId),
      itemId = event.itemId,
      quantity = event.quantity)
  }

  private def transformItemAdjusted(
      envelope: EventEnvelope[ShoppingCart.ItemQuantityAdjusted])
      : proto.ItemQuantityAdjusted = {
    val event = envelope.event
    proto.ItemQuantityAdjusted(
      cartId = PersistenceId.extractEntityId(envelope.persistenceId),
      itemId = event.itemId,
      quantity = event.newQuantity)
  }

  private def transformItemRemoved(
      envelope: EventEnvelope[ShoppingCart.ItemRemoved]): proto.ItemRemoved = {
    val event = envelope.event
    proto.ItemRemoved(
      cartId = PersistenceId.extractEntityId(envelope.persistenceId),
      itemId = event.itemId)
  }

  private def transformCheckedOut(
      envelope: typed.EventEnvelope[ShoppingCart.CheckedOut])
      : proto.CheckedOut =
    proto.CheckedOut(PersistenceId.extractEntityId(envelope.persistenceId))

  
  
}

